/**
   File name: encounterManager.cpp
   Ex number: Project, part 2
   Login: adiluria
   Name: Adi Luria
   ID: 017651662
*/

#include "encounterManager.h"
#include "student.h"
#include "teacher.h"
#include "flower.h"
#include "seed.h"
#include "mole.h"
#include "givatRamSimulation.h"

EncounterManager::EncounterManager(GivatRamSimulation& sim, 
				   unsigned int flowersRisked):
  _sim(sim),_flowersRisked(flowersRisked){
  
  // initing the EVEN-NIAR game map
  _game[Student::_FIRST_WEAPON]=EVEN_VALUE;
  _game[Student::_SECOND_WEAPON]=NIAR_VALUE;
  _game[Student::_THIRD_WEAPON]=MIS_VALUE;
}

void EncounterManager::encounterStudentAndTeacher(Student* s, Teacher* t){
  if(s->isAlive()){
    unsigned int studFlow = s->getNumFlowers();
    unsigned int teachFlow = t->getNumFlowers();
    unsigned int greed = t->getFlowerGreed();

    if(studFlow <= greed){
      t->setNumFlowers(teachFlow + studFlow);
      s->setNumFlowers(0);

      // student dies for lack of flowers
      _sim.getWorld().kill(s);

      //s->setAliveness(false);
    }
    else{
      t->setNumFlowers(teachFlow + greed);
      s->setNumFlowers(studFlow - greed);
    }
    _sim.dump();
  }
}

void EncounterManager::encounterStudentAndStudent(Student* s1, Student* s2){
  if(s1->isAlive() && s2->isAlive()){
    int result = play(s1->getGameStrategy(),s2->getGameStrategy());
    if(result>0){ // first stud has won
      takeFlowers(s1,s2);
    }
    else if(result<0){ // second stud has won
      takeFlowers(s2,s1);
    }
    _sim.dump();
  }
}

// the EVEN NIAR game uses the addition of two int values of strategies. 
// the values are extracted from the _game map, according to the name of 
// the strategy.
// the addition is compared to the values in the enum Game, and 
// with a nested if determines the winner.
int EncounterManager::play(const string& weap1, const string& weap2){
  unsigned int w1 = _game[weap1];
  unsigned int w2 = _game[weap2];
  
  if(w1==w2) // both have same game strategy
    return 0;

  // all nested ifs check only if w1 is the winning strategy.
  // if it isn't, the function will return -1 by the end, meaning the 
  // winning strategy is w2.
  else if(w1+w2 == NIAR_AND_EVEN){
    if(w1 == NIAR_VALUE) 
      return 1;
  }
  
  else if(w1+w2 == EVEN_AND_MIS){
    if(w1 == EVEN_VALUE)
      return 1;
  }

  else if(w1+w2 == MIS_AND_NIAR){
    if(w1 == MIS_VALUE)
      return 1;
  }
  return -1; // the 2nd strategy wins
}

void EncounterManager::takeFlowers(Student* winner, Student* loser){
  unsigned int loserNum = loser->getNumFlowers();
  if(loserNum <= _flowersRisked){
    winner->setNumFlowers(winner->getNumFlowers() + loserNum);
    loser->setNumFlowers(0);
  }
  else{
    winner->setNumFlowers(winner->getNumFlowers() + _flowersRisked);
    loser->setNumFlowers(loserNum - _flowersRisked);
  } 
}


