/**
   File name: encounterManager.h
   Ex number: Project, part 2
   Login: adiluria
   Name: Adi Luria
   ID: 017651662

   File description: This class defines an encounterManager. It's
   purpose is to gather all functions that are created as a result of
   an encounter.
*/

#ifndef _ENCOUNTER_MANAGER_H
#define _ENCOUNTER_MANAGER_H

#include <map>
#include <string>

class GivatRamSimulation;
class Student;
class Teacher;
class Flower;
class Seed;
class Mole;

class EncounterManager{
 public:
  
  /* ctor, the args are the ref to the releant simulation and the 
   number of flowers risked in the EVEN-NIAR game. */
  EncounterManager(GivatRamSimulation& sim, unsigned int flowersRisked);
  
  /* dtor */
  ~EncounterManager(){}
  
  /* an encounter between a mole-cricket and flower which causes an eating 
     event where the mole cricket eats the flower. */
  void encounterMoleCricketAndFlower(Mole* m, Flower* f){}
  
  /* this encounter causes the two students to embark in a game of
   EVEN-NIAR, where the winner is awarded the number of flowers given 
   in the ctor.*/
  void encounterStudentAndStudent(Student* s1, Student* s2);
  
  /* causes the student to give the teacher the number of flowers as 
   given by the "flower-greed" param of the teacher. Should the student 
   lose all his flowers - he dies. */
  void encounterStudentAndTeacher(Student* s, Teacher* t);
  
  /* this encounter causes a picking event, in which the student picks up 
     the flower if the flower is still alive by the end of the picking time
     and the student is still in the relevant cell.*/
  void encounterStudentAndFlower(Student* s, Flower* f){}
  
  /* sets the number of flowers which are risked in the EVEN-NIAR game. */
  void setFlowersRisked(unsigned int flowersRisked) {
	_flowersRisked = flowersRisked;
  }
  
  /* returns the number of flowers which are risked in the EVEN-NIAR game.*/ 
  unsigned int getFlowersRisked()const {return _flowersRisked;}

 private:
  // represents the ref to the simulation. 
  GivatRamSimulation& _sim; 
  
  // denotes the num of flowers risked in a game of EVEN-NIAR
  unsigned int _flowersRisked;
  
  // the map repres the game "weapons" and their int val.
  map<string,unsigned int> _game;

  // the following enum repres the weapons in the game and the possible 
  // sums of the weapon-values. no two sums can be the same, due to the 
  // choice of values for the different strategies.
  //for elaborations, see "play" member-fun. in the cpp file.
  enum Game{EVEN_VALUE=1, NIAR_VALUE=2, MIS_VALUE=4, NIAR_AND_EVEN=3, 
	    EVEN_AND_MIS=5, MIS_AND_NIAR=6};

  /* returns 0 if both weapons are alike, 1 if the first wins, -1 if the 
     first loses */
  int play(const string& w1, const string& w2);

  /* private method in which the giver gives the taker "flowers risked" 
   or all his flowers should he not have enough. */
  void takeFlowers(Student* winner, Student* loser); 
};
#endif

