/**
   File name: entity.h
   Ex number: Project, part 1
   Login: adiluria
   Name: Adi Luria
   ID: 017651662

   File description: A virtual class which represents the base class
   of the Entity hierarchy. Every entity is defined by the 3 params:
   location on the Givat-Ram grid, and its order in the list of
   entities. The main function it offers are the encounter methods
   and an ostream representing the string representation of the entity.
   The encounter functions use the double dispatch concept.
*/

#ifndef _ENTITY_H
#define _ENTITY_H

/*--------------------- include files -------------------------------*/
#include <iostream.h>
#include <string>
#include "checks.h"
#include "illegalEntityException.h"
#include "point.h"
#include "encounterManager.h"

/*------------------------ forward declarations --------------------*/
class Student;
class Teacher;
class Flower;
class Seed;
class Mole;

/*---------------------- class Entity ------------------------------*/
class Entity{
 public:
      /*------------------ ctors -----------------------------------*/

      /* default ctor. inits the Entity with the default values of
	 the fields. */
      Entity():_isAlive(true),_isBusy(false){}

      /* constructs the entity with its position and order in the
	 list of entities in the Givat Ram campus. */
      Entity(unsigned int order, unsigned int xPos, unsigned int yPos);

      /*--------------------- dtor ---------------------------------*/
      virtual ~Entity(){}

      /*----------------- accessories ------------------------------*/

      /* returns the order of the entity in the entity list. */
      unsigned int getOrder()const {return _order;}

      /* returns the x position of the entity in the grid. */
      unsigned int getXPos()const {return _xPos;}

     /* returns the y position of the entity in the grid. */
      unsigned int getYPos()const {return _yPos;}

      /* sets the order of the entity in the entity list. */
      void setOrder(unsigned int order) {_order = order;}

      /* sets the x position of the entity.*/
      void setXPos(unsigned int xPos);

      /* sets the y position of the entity.*/
      void setYPos(unsigned int yPos);
      
      /* sets the entity's aliveness flag. */
      void setAliveness(bool alive){_isAlive = alive;}

      /* returns true if the entity is alive, false otherwise. */
      bool isAlive() const {return _isAlive;}

      /* sets the entity's business flag */
      void setBusiness(bool busy){_isBusy = busy;}

      /* returns true if the entity is busy, flase otherwise. */
      bool isBusy() const {return _isBusy;}
      
      /* returns the string representation of the entity in the
       format of an ostream. it is appended to the given ostream.
       */
      virtual ostream& stringFormat(ostream& out = cout)const;

      /* defines an encounter between two entities. uses the
	 double dispatch concept in order to include all
	 possible encounters.
       */
      virtual void encounter(Entity* ent, EncounterManager& enc)=0;

      /* encounter between this entity and a student. */
      virtual void encounterStudent(Student* s,
				    EncounterManager& enc)=0;

      /* encounter between this entity and a teacher. */
      virtual void encounterTeacher(Teacher* t,
				    EncounterManager& enc)=0;

      /* encounter between this entity and a flower. */
      virtual void encounterFlower(Flower* f,
				   EncounterManager& enc)=0;

      /* encounter between this entity and a seed. */
      virtual void encounterSeed(Seed* s,
				 EncounterManager& enc)=0;

      /* encounter between this entity and a mole. */
      virtual void encounterMole(Mole* m,
				 EncounterManager& enc)=0;

      /*---------------------- consts -------------------------------------*/
      // defines the string repres. of the fields of the entity
      static const string _ORDER;
      static const string _XPOS;
      static const string _YPOS;
      
      // defines the chars used in the output representation of the entities.
      static const string _SPACE;
      static const string _EQUALS;
      static const string _OPEN;
      static const string _CLOSE;
      
 protected:

      // denotes the order of the entity in the list
      unsigned int _order;

      // denotes the x position in the grid
      unsigned int _xPos;

      //denotes the y position in the grid
      unsigned int _yPos;

      // denotes whether the entity is alive or not
      bool _isAlive;

      // denotes whether the entity is busy or not 
      bool _isBusy;
};

#endif

/*************************************************************************
			       EOF
**************************************************************************/

