/**
   File name: factory.cpp
   Ex number: Project, part 2
   Login: adiluria
   Name: Adi Luria
   ID: 017651662
*/

#include "factory.h"

Factory::Factory(){

      //inits the map of creator functions
      _functions[Student::_STUDENT] = createStudent;
      _functions[Teacher::_TEACHER] = createTeacher;
      _functions[Flower::_FLOWER] = createFlower;
      _functions[Seed::_SEED] = createSeed;
      _functions[Mole::_MOLE] = createMole;
}

// returns the entity created on the heap
Entity* Factory::init(strMap s, unsigned int theOrder){
      // finds the name of the command in the string map
      string key = s[Parser::_FIRST_PARAM];
      // locates the entity type in the function map
      funMap::const_iterator itr =  _functions.find(key);
      if(itr == _functions.end()){

	     throw IllegalFormatException
	       ("This is not a legal command name.");

	     // should be:	   
	     //throw IllegalFormatException();
	     
	    
      }
      MapReader read(s);
      // initing the ptr to function.
      PtrToFun create;
      create =_functions[key];
      return(create(theOrder,read));
}

void entityCreator(Entity* ent, unsigned int theOrder,MapReader& read){
      ent->setOrder(theOrder);
      ent->setXPos(read.getFromMap<unsigned int>(Entity::_XPOS));
      ent->setYPos(read.getFromMap<unsigned int>(Entity::_YPOS));
}

void movableEntityCreator(MovableEntity* mover, MapReader& read){
  try{
    mover->setMoveTime(read.getFromMap<float>(Movable::_MOVE_TIME));
    mover->setMovePattern(read.getFromMap<string>(Movable::_MOVE_PAT));
  }catch(IllegalEntityException& iee){
    throw IllegalEntityException(mover->getOrder());
  }
}

void plantCreator(PlantFamily* ple, MapReader& read){
      ple->setLifetime(read.getFromMap<float>(PlantFamily::_LIFETIME));
      ple->setSeedingTime(read.getFromMap<float>(PlantFamily::_SEED_TIME));
      ple->setAirialTime(read.getFromMap<float>(PlantFamily::_AIRIAL));
      ple->setFloweringTime(read.getFromMap<float>(PlantFamily::_FLOW_TIME));
}

// returns a new student object on the heap
Entity* createStudent(unsigned int theOrder, MapReader& read){
      Student* newStud = 0;
      try{
	newStud = new Student;
	    if(read.size() != Factory::_STUD_NUM_PARAMS){
		  throw ReaderException();
		  //throw IllegalFormatException
		  //	("Not the right num of params for student.");
	    }
	    entityCreator(newStud,theOrder,read);
	    movableEntityCreator(newStud,read);
	    newStud->setNumFlowers
		  (read.getFromMap<unsigned int>(Student::_STUD_NUM_FLOW));
	    newStud->setPickTime(read.getFromMap<float>(Student::_PICK_TIME));
	    newStud->setGameStrategy
		  (read.getFromMap<string>(Student::_GAME_STRAT));
	    return newStud;
      }catch(ReaderException& re){
	    delete newStud; // rollback
	    throw IllegalEntityException(theOrder);
      }catch(bad_alloc){
	delete newStud;
	cerr<<"The program has run out of memory."<<endl;
      }
}

// returns a new teacher object on the heap
Entity* createTeacher(unsigned int theOrder, MapReader& read){
      Teacher* newTeach = 0;
      try{
	newTeach = new Teacher;
	    if(read.size() != Factory::_TEACH_NUM_PARAMS){
		  throw ReaderException();
		  //throw IllegalFormatException
		  //	("Not the right num of params for teacher.");
	    }
	    entityCreator(newTeach,theOrder,read);
	    movableEntityCreator(newTeach,read);
	    newTeach->setNumFlowers
		  (read.getFromMap<unsigned int>(Teacher::_TEACH_NUM_FLOW));
	    newTeach->setFlowerGreed
		  (read.getFromMap<unsigned int>(Teacher::_FLOW_GREED));
	    return newTeach;
      }catch(ReaderException& re){
	    delete newTeach; // rollback
	    throw IllegalEntityException(theOrder);
      }catch(bad_alloc){
	delete newTeach;
	cerr<<"The program has run out of memory."<<endl;
      }
}

// returns a new flower object on the heap
Entity* createFlower(unsigned int theOrder, MapReader& read){
      Flower* newFlow = 0;
      try{
	newFlow = new Flower;
	    if(read.size() != Factory::_FLOW_NUM_PARAMS){
		  throw ReaderException();
		  //throw IllegalFormatException
		  //    ("Not the right num of params for flower.");
	    }
	    entityCreator(newFlow,theOrder,read);
	    plantCreator(newFlow,read);
	    return newFlow;
      }catch(ReaderException& re){
	    delete newFlow; // rollback
	    throw IllegalEntityException(theOrder);
      }catch(bad_alloc){
	delete newFlow;
	cerr<<"The program has run out of memory."<<endl;
      }
}

// returns a new flower-seed object on the heap
Entity* createSeed(unsigned int theOrder, MapReader& read){
      Seed* newSeed = 0;
      try{
	newSeed = new Seed;
	    if(read.size()!= Factory::_SEED_NUM_PARAMS){
		  throw ReaderException();
		  //throw IllegalFormatException
		  //	("Not the right num of params for flower-seed.");
	    }
	    entityCreator(newSeed,theOrder,read);
	    plantCreator(newSeed,read);
	    return newSeed;
      }catch(ReaderException& re){
	    delete newSeed; // rollback
	    throw IllegalEntityException(theOrder);
      }catch(bad_alloc){
	delete newSeed;
	cerr<<"The program has run out of memory."<<endl;
      }
}

// returns a new mole-cricket object on the heap
Entity* createMole(unsigned int theOrder, MapReader& read){
      Mole* newMole = 0;
      try{
	newMole = new Mole;
	    if(read.size()!= Factory::_MOLE_NUM_PARAMS){
		  throw ReaderException();
		  //throw IllegalFormatException
		  //    ("Not the right num of params for mole-cricket.");
	    }
	    entityCreator(newMole,theOrder,read);
	    movableEntityCreator(newMole,read);
	    newMole->setEatingTime(read.getFromMap<float>(Mole::_EAT_TIME));
	    return newMole;
      }catch(ReaderException& re){
	    delete newMole; // rollback
	    throw IllegalEntityException(theOrder);
      }catch(bad_alloc){
	delete newMole;
	cerr<<"The program has run out of memory."<<endl;
      }
}

const unsigned int Factory::_STUD_NUM_PARAMS = 8;
const unsigned int Factory::_TEACH_NUM_PARAMS = 7;
const unsigned int Factory::_FLOW_NUM_PARAMS = 7;
const unsigned int Factory::_SEED_NUM_PARAMS = 7;
const unsigned int Factory::_MOLE_NUM_PARAMS = 6;

/*********************************************************************
				EOF
**********************************************************************/
