/**
   File name: givatRamSimulation.cpp
   Ex number: Project, part 2
   Login: adiluria
   Name: Adi Luria
   ID: 017651662
*/

#include "givatRamSimulation.h"

void GivatRamSimulation::init(){
  try{
    Parser parse(_in);
    strMap exec = parse.getBlock();
    MapReader read(exec);
    extractData(read);
    
    // schedualing the first wind event
    WindEvent* wevent = 0;
    try{
      wevent = new WindEvent(_now+_world.getWind().getMoveTime());
    }catch(bad_alloc){
      delete wevent;
      cerr<<"No more memory for the program."<<endl;
    }
    schedule(wevent);
    
    while(true){
      try{
	++_order;
	exec = parse.getBlock();
	Entity* ent = _factory.init(exec, _order);
	Point p(ent->getXPos(), ent->getYPos());
	
	//add the new entity to the world
	_world.addEntity(ent);
	
	//inits the relevant events
	MovableEntity* mover  = dynamic_cast<MovableEntity*>(ent);
	if(mover){
	  scheduleBasics(mover);
	}
	
	Flower* flow = dynamic_cast<Flower*>(ent);
	if(flow)
	  scheduleBasics(flow);
	
	Seed* seed = dynamic_cast<Seed*>(ent);      
	if(seed){
	  scheduleBasics(seed);
	}
      }catch(IllegalEntityException& iee){
	    cerr<<iee.getMessage()<<endl;
      }
    }
  }
  catch(EndOfInput& end){} // file was rad completely
  //catch(Exception& e){cout<<"in special catch"<<endl;}
//   throw IllegalFormatException("gili");
//   }
}


void GivatRamSimulation::run(){
      Simulation::run(_creationTime, _endTime);
}

void GivatRamSimulation::scheduleBasics(MovableEntity* mover){
  Event* event = 0;
  try{
    event = new MovingEvent(_now+mover->getMoveTime(),mover->getOrder());
  }catch(bad_alloc){
    delete event;
    cerr<<"Memory has run out."<<endl;
  }
  schedule(event);
}

void GivatRamSimulation::scheduleBasics(Flower* flow){
  unsigned int numEvents = 3;
  Event* events[numEvents];
  // initing all ptrs to null
  for(unsigned int i=0; i<numEvents; ++i)
    events[i] = 0;
  
  try{
    // the flower seeds twice after it is born
    events[0] = new BirthEvent(_now+flow->getSeedingTime(),_order);
    events[1] = new BirthEvent(_now+flow->getSeedingTime(),_order);
    
    // the flower dies after its lifetime
    events[2] = new DeathEvent(_now+flow->getLifetime(),_order);
  }catch(bad_alloc){
    // deleting all events created with the new operator, memory failure
    for(unsigned int i=0; i<numEvents; ++i)
      delete events[i];
    cerr<<"Memory has run out."<<endl;
  }
  // schedule all events in the array
  for(unsigned int i=0; i<numEvents; ++i)
    schedule(events[i]);
}

void GivatRamSimulation::scheduleBasics(Seed* seed){
  Event* event = 0;

  try{
    // seed lands
    event = new LandingEvent(_now+seed->getAirialTime(),_order);

  }catch(bad_alloc){
	delete event;
	cerr<<"Memory has run out."<<endl;
  }
  schedule(event);
  
  // adding the flying object to the flyers list
  _world.getWind().addFlyer(seed->getOrder());
}

void GivatRamSimulation::extractData(MapReader& read){
  try{
    _creationTime = read.getFromMap<float>(Parser::_THIRD_PARAM);
    _now = _creationTime; // initing the time "now"
    _endTime  = read.getFromMap<float>(Parser::_FOURTH_PARAM);
    _flowersRisked = 
      read.getFromMap<unsigned int>(Parser::_SEVENTH_PARAM);
	    
    // only the encounter manager needs to know how many flowers are 
    // risked in the game.
    _manager.setFlowersRisked(_flowersRisked); 

    // params for the world object
    _xSize = read.getFromMap<unsigned int>(Parser::_FIRST_PARAM);
    _ySize = read.getFromMap<unsigned int>(Parser::_SECOND_PARAM);
    _moveTime = read.getFromMap<float>(Parser::_FIFTH_PARAM);
    _movePattern = read.getFromMap<string>(Parser::_SIXTH_PARAM);
	    
    _world = World(_xSize, _ySize, _moveTime, _movePattern);
  }catch(ReaderException& re){
    throw IllegalParamException
      ("The given params for the world aren't correct.");
  }
}

ostream& GivatRamSimulation::dump(){
  ostream& temp =  _out << _DUMP << _now << _NEWLINE << _xSize << _SPACE 
			<< _ySize << _SPACE << _creationTime << _SPACE 
			<< _endTime << _SPACE << _moveTime << _SPACE 
			<< _movePattern << _SPACE << _flowersRisked << _NEWLINE;
  
  // concating the entity repres as returned from the world.
 return _world.dump(temp);
}

const string GivatRamSimulation::_DUMP = "# World dump at ";
const string GivatRamSimulation::_SPACE = " "; 
const string GivatRamSimulation::_NEWLINE = "\n";


/************************************************************************
			       EOF
*************************************************************************/
