/**
   File name: givatRamSimulation.h
   Ex number: Project, part 2
   Login: adiluria
   Name: Adi Luria
   ID: 017651662
   
   File description:
 
*/

#ifndef _GIVAT_RAM_SIMULATION_H
#define _GIVAT_RAM_SIMULATION_H

#include "simulation.h"
#include "event.h"
#include "movingEvent.h"
#include "birthEvent.h"
#include "deathEvent.h"
#include "landingEvent.h"
#include "windEvent.h"
#include "factory.h"
#include "parser.h"
#include "mapReader.h"
#include "encounterManager.h"

class GivatRamSimulation: public Simulation{
 public:
  GivatRamSimulation(istream& in = cin,ostream& out = cout):
    _in(in),_out(out),_order(0),_flowersRisked(0),
    _manager(*this,_flowersRisked){
    init(); 
  }
  
  void run(); 
  
  virtual void init();
  virtual void update() {_world.buryAll();}
  virtual void terminate(){}

  void scheduleBasics(MovableEntity* mover);
  void scheduleBasics(Flower* flow);
  void scheduleBasics(Seed* seed);

  World& getWorld() {return _world;}

  ostream& dump();

  EncounterManager& getEncounterManager() {return _manager;}

  unsigned int getNextOrder() {return ++_order;}

private:
  // iostreams
  istream& _in;
  ostream& _out;
  // the order of the entity currently being created
  unsigned int _order;

  Factory _factory;
  World _world;
  
  // params for the world
  unsigned int _xSize;
  unsigned int _ySize;
  float _moveTime;
  string _movePattern;

  // begin and end times of the simulation
  float _creationTime;
  float _endTime;

  // num of flowers risked in game
  unsigned int _flowersRisked;
  
  EncounterManager _manager;

  /* private metod used to initiate the world params in the simulation. */
  void extractData(MapReader& read);
 
  static const string _DUMP;
  static const string _SPACE;
  static const string _NEWLINE;
};
#endif
