/**
   File name: grid.h
   Ex number: Project, part 2
   Login: adiluria
   Name: Adi Luria
   ID: 017651662

   File description: This class defines a Grid, and is a map connecting 
   between a location Point and a template list.
   The class offers iterators on a list of a specific Point, and also 
   iterators on the whole grid.
*/

#ifndef _GRID_H
#define _GRID_H

#include <map> 
#include <set> 
#include <string> 
#include <strstream> 
#include "point.h" 
#include "lessComparator.h"

template <typename T>
class Grid{ 
 
 public: 

  typedef set<T> Set;
  typedef LessComparator<Point> pCmp; 
  typedef map<Point,Set,pCmp> Board;
  typedef Board::const_iterator const_grid_iterator;
  typedef Board::iterator grid_iterator;
  typedef Set::iterator iterator;
  typedef Set::const_iterator const_iterator;
      
  /*********************** ctor ***************************************/ 
  /* default ctor */ 
  Grid(){} 
  
  /* dtor */ 
  ~Grid(){} 
  
  /* returns an iterator to the beginning of the grid. */
  const_grid_iterator all(){
    return(_board.begin());
  }

  /* returns an iterator to the end of the grid. */
  const_grid_iterator endAll(){
    return(_board.end());
  }

  /* adds a type T to the Board, into the list connected to the given 
     point. if the point does not exist in the map, it is added into
     it and an empty list is paired to it. */
  void addType(const Point& p, const T& t){
	    
    // the object which is constructed by default in the map      
    Board::data_type def;  
    if(_board[p]==def){
      Set st; 
      _board[p] = st; 
    }
    _board[p].insert(t);
  }
  
  /* removes a specified type from the set attached to the given
     point. */
 void removeType(const Point& p, const T& t){
   
   grid_iterator itr = _board.find(p);
   (itr->second).erase(t);
   if((itr->second).empty())
     _board.erase(itr->first);
 }

 /* moves a type from one point to the other. */
 void moveType(const Point& from, const Point& to, const T& t){
   removeType(from,t);
   addType(to,t);
 }
 
 /* returns an iterator to the beggining of the set of types which is
	 the pair of the given point. */ 
 const_iterator begin(const Point& p)const{
   const_grid_iterator itr = _board.find(p); 
 
   if((itr->second).empty())
     return((itr->second).end());
   return((itr->second).begin()); 
 }
 
 /* returns an iterator the the end of the list of the given point */
 const_iterator end(const Point& p)const{
   const_grid_iterator itr = _board.find(p);
   return((itr->second).end());
 }
 
 private: 
 // denotes the map which this grid consists of 
 Board _board;  
}; 
#endif 

/***********************************************************************
			       EOF
 ************************************************************************/
