/**
   File name: parser.h
   Ex number: Project, part 1
   Login: adiluria
   Name: Adi Luria
   ID: 017651662

   File description:
*/

#ifndef _PARSER_H
#define _PARSER_H

/*------------------------------ include files ------------------------*/
#include <string>
#include <iostream.h>
#include <map>
#include <strstream>
#include "endOfInput.h"
#include "illegalFormatException.h"
#include "illegalParamException.h"

/*------------------------------ typdef -------------------------------*/
typedef map<string, string> strMap;

class Parser{
 public:
  
  /*---------------------- ctor -----------------------------------*/
  /* construcs a parser on a given input stream. inits the end-of-file
     flag to be false. */
  Parser(istream& in):_input(in),_finishedBlock(false),_firstRead(false){}

  /*----------------------- dtor ----------------------------------*/
  ~Parser(){}

  /* the actual function which causes the Parser to parse the object.
	 it reades one block at a time, and adjusts itself to read either
	 the entity list or the encounter/function list. all the params of the
	 commands are returned in the string map: the creator block includes
	 the params and the values plus the entity type name; the function 
	 block contains the function name and the two entities which will be 
	 involved in that function.
       */
  strMap getBlock();


  /*------------------------------- consts ------------------------------*/
// consts used for the commands in the map that will be returned
  static const string _FIRST_PARAM;
  static const string _SECOND_PARAM;
  static const string _THIRD_PARAM;
  static const string _FOURTH_PARAM;
  static const string _FIFTH_PARAM;
  static const string _SIXTH_PARAM;
  static const string _SEVENTH_PARAM;


 private:
  // denotes the ref to the input stream which will be parsed.
  istream& _input;
  // true if the current block of the file has been read, false otherwise.
  bool _finishedBlock;

      // true if first block in input has been read.
  bool _firstRead;

  /* gets the next string if the end of the file hasn't been reached.
       throws a good exception if the file has ended correctly after the
       end-of-list flag, and throws an illegal format exception if
      the file has ended inclorrectly.*/
  string next();

  /* returns whether the end of the file was reached, not counting
	 the white spaces */
  bool endOfFile() const;
  
  // consts representing the delims and flags in the input stream.
  static const string _FIRST_DELIM;
  static const string _LAST_DELIM;
  static const string _EQUALS_SIGN;
  static const string _ROW_DELIM;
  static const char _SPACE;
  static const char _NEWLINE;
  static const char _TAB;
};
#endif

/***********************************************************************
				   EOF
************************************************************************/

