/**
   File name: windEvent.cpp
   Ex number: Project, part 2
   Login: adiluria
   Name: Adi Luria
   ID: 017651662
*/

#include "windEvent.h"

void WindEvent::process(Simulation& sim){

  GivatRamSimulation& gsim = dynamic_cast<GivatRamSimulation&> (sim);
  World& world = gsim.getWorld(); 
  Wind& wind = world.getWind();

  char nextDir = wind.getNextDirection();
  
  // iterators on the flyers in the world
  Wind::iterator first = wind.begin();
  Wind::iterator last = wind.end();

  // iterating on the flyers DB
  for(; first!=last; ++first){
    Entity* ent = world.getEntity(*first);
    if(ent!=0){ // entity exists in the database
      
      // flyer has just been added to the flyers list, hasn't been 
      // alive the entire previous wind movement:
      if(!ent->isBusy())
	ent->setBusiness(true);
      
      else{ // the entity will be moved
	passiveMove(gsim, ent, nextDir);
      }
    }
  }
  
  gsim.dump(); // dumping after the win has moved all flyers

  WindEvent* wevent = 0;
  try{
    wevent = new WindEvent(gsim.time()+wind.getMoveTime());
  }catch(bad_alloc){
    delete wevent;
    cerr<<"This program has no more memory."<<endl;
  }
  gsim.schedule(wevent);
}

void WindEvent::passiveMove(GivatRamSimulation& gsim, Entity* ent, char dir){
  World& world = gsim.getWorld(); 
  Wind& wind = world.getWind();
  Point p = wind.getNextLocation(ent->getXPos(),ent->getYPos(),dir);
    
  // the length and width of the world
  unsigned int xSize = world.getXSize();
  unsigned int ySize = world.getYSize();

  // the new coordinates
  unsigned int newX = p.getX();
  unsigned int newY = p.getY();

  // setting the new point to be within the world's limits
  if(newX < World::_MIN_COORD)
    ent->setXPos(xSize);
  else if(newY < World::_MIN_COORD)
    ent->setYPos(ySize);
  else if(newX > xSize)
    ent->setXPos(World::_MIN_COORD);
  else if(newY > ySize)
    ent->setYPos(World::_MIN_COORD);
  else{
    ent->setXPos(newX);
    ent->setYPos(newY);
  }
}

const unsigned int WindEvent::_WIND_ORDER = 0;
