/**
   File name: world.cpp
   Ex number: Project, part 2
   Login: adiluria
   Name: Adi Luria
   ID: 017651662
*/

#include "world.h"

Entity* World::getEntity(unsigned int order)const{
      const_iterator itr = _entMap.find(order);
      // the given order was not found in the EntityMap
      if(itr == _entMap.end())
	    return(0);
      // returning ptr to Entity
      return(itr->second);
}

void World::addEntity(Entity* ent){
      unsigned int order = ent->getOrder();
      unsigned int xpos = ent->getXPos();
      unsigned int ypos = ent->getYPos();

      // checking bounds of entity in grid
      if((xpos < _MIN_COORD) || (xpos > _xSize) ||
	 (ypos < _MIN_COORD) || (ypos > _ySize))
	    throw IllegalEntityException(order);
      _entMap[order] = ent;
      _grid.addType(Point(xpos,ypos), order);
}

void World::removeEntity(unsigned int order){
      Entity* ent = getEntity(order);
      
      // if(ent!=0){

      // removing from the location grid
      _grid.removeType(Point(ent->getXPos(),ent->getYPos()),order);
      
      // removing from the entity map
      _entMap.erase(order);
      delete ent;
      ent = 0;

      // }
}

void World::changePoint(MovableEntity* ent){
      unsigned int xpos = ent->getXPos();
      unsigned int ypos = ent->getYPos();
      unsigned int order = ent->getOrder();
      _grid.removeType(Point(xpos,ypos),order);
      Point p = ent->getNextLocation(xpos, ypos, ent->getNextDirection());
      unsigned int newX = p.getX();
      unsigned int newY = p.getY();

      // setting the new point to be within the world's limits
      if(newX < _MIN_COORD)
	    ent->setXPos(_xSize);
      else if(newY < _MIN_COORD)
	    ent->setYPos(_ySize);
      else if(newX > _xSize)
	    ent->setXPos(_MIN_COORD);
      else if(newY > _ySize)
	    ent->setYPos(_MIN_COORD);
      else{
	    ent->setXPos(newX);
	    ent->setYPos(newY);
      }
}

void World::encounterAll(Entity* ent, EncounterManager& manager){
  const_location_iterator first = begin(ent);
  const_location_iterator last = end(ent);
  
      for(;first!=last;++first){
	    Entity* temp = getEntity(*first);
	    if(temp->isAlive())
		  ent->encounter(temp,manager);
      }
}

void World::addToPoint(Entity* ent){
  _grid.addType(Point(ent->getXPos(),ent->getYPos()),ent->getOrder());
}


// void World::moveAndEncounter(MovableEntity* ent, EncounterManager& manager){
//   unsigned int xpos = ent->getXPos();
//       unsigned int ypos = ent->getYPos();
//       unsigned int order = ent->getOrder();
//       // removing from the previous location
//       _grid.removeType(Point(xpos,ypos),order);
//       Point p = ent->getNextLocation(xpos,ypos); // next location
      
//       const_location_iterator first = begin(p);
//       const_location_iterator last = end(p);
      
//       // encountering all entities in the location
//       for(;first!=last;++first){
// 	    Entity* temp = getEntity(*first);
// 	    if(temp->isAlive())
// 	      ent->encounter(temp,manager);
//       }

//       // next coordinates
//       unsigned int newX = p.getX();
//       unsigned int newY = p.getY();
      
//       // setting the new point to be within the world's limits
//       if(newX < _MIN_COORD)
// 	    ent->setXPos(_xSize);
//       else if(newY < _MIN_COORD)
// 	    ent->setYPos(_ySize);
//       else if(newX > _xSize)
// 	    ent->setXPos(_MIN_COORD);
//       else if(newY > _ySize)
// 	    ent->setYPos(_MIN_COORD);
//       else{
// 	    ent->setXPos(newX);
// 	    ent->setYPos(newY);
//       }

//       // adding the entity to the new location after it has been formatted to
//       // fit the grid
//       _grid.addType(Point(ent->getXPos(),ent->getYPos()),order);
// }

void World::buryAll(){
  while(!_tempDead.empty()){
    unsigned int killed = _tempDead.front();
    removeEntity(killed);
    _tempDead.pop();
  }
}

// a method which will pop the entities from the dead stack and then call
// the remove entity method.

ostream& World::printWorld(ostream& out = cout){
      
      Grid::const_grid_iterator beg = _grid.all();
      Grid::const_grid_iterator en = _grid.endAll();
      for(;beg!=en;++beg){
	    Point p = beg->first;
	    out<<p<< ": ";
	    Grid::const_iterator setBeg = _grid.begin(p);
	    Grid::const_iterator setEn = _grid.end(p);
	    for(;setBeg!=setEn;++setBeg){
	      (getEntity(*setBeg))->stringFormat(out);  
	    }
	    out<<endl;
      }
      return out;
}

ostream& World::dump(ostream& out){
  const_iterator first = _entMap.begin();
  const_iterator last = _entMap.end();
  for(; first!=last; ++first){
    (first->second)->stringFormat(out);
  }
  return out;
}

const int World::_MIN_COORD = 1;

/********************************************************************
				  EOF
*********************************************************************/
